function HttpError(code, message) {
  this.code = code || 0
  this.message = message || ''
}

HttpError.prototype = Error.prototype

function isEmpty(val) {
  return val === undefined || val === null || (typeof val === 'number' && isNaN(val))
}

function cleanEmptyValue(obj) {
  const newObj = {}
  for (const key in obj) {
    const val = obj[key]
    if (!isEmpty(val)) {
      newObj[key] = val
    }
  }
  return newObj
}

function apiFactory(actions) {
  const apis = {}
  actions.forEach((action) => {
    apis[action] = ({ apig, ...inputs }) => {
      inputs = cleanEmptyValue(inputs)
      return new Promise((resolve, reject) => {
        apig.request(
          {
            Action: action,
            RequestClient: 'ServerlessComponent',
            Token: apig.defaults.Token || null,
            ...inputs
          },
          function(err, data) {
            if (err) {
              return reject(err)
            } else if (data.code !== 0) {
              return reject(new HttpError(data.code, data.message))
            }
            resolve(data.data)
          }
        )
      })
    }
  })

  return apis
}

const ACTIONS = [
  'AddCdnHost',
  'SetHttpsInfo',
  'GetHostInfoByHost',
  'DeleteCdnHost',
  'OfflineHost',
  'UpdateCdnConfig'
]
const APIS = apiFactory(ACTIONS)

module.exports = APIS
